@@ -239,7 +239,9 @@ Tk_PackObjCmd(
 	if (TkGetWindowFromObj(interp, tkwin, objv[2], &tkwin2) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	prevPtr = GetPacker(tkwin2);
+	if (!(prevPtr = GetPacker(tkwin2))) {
+	    return TCL_OK;
+	}
 	if (prevPtr->containerPtr == NULL) {
 	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		    "window \"%s\" isn't packed", argv2));
@@ -256,7 +258,9 @@ Tk_PackObjCmd(
 	if (TkGetWindowFromObj(interp, tkwin, objv[2], &tkwin2) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	containerPtr = GetPacker(tkwin2);
+	if (!(containerPtr = GetPacker(tkwin2))) {
+	    return TCL_OK;
+	}
 	prevPtr = containerPtr->contentPtr;
 	if (prevPtr != NULL) {
 	    while (prevPtr->nextPtr != NULL) {
@@ -273,7 +277,9 @@ Tk_PackObjCmd(
 	if (TkGetWindowFromObj(interp, tkwin, objv[2], &tkwin2) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	packPtr = GetPacker(tkwin2);
+	if (!(packPtr = GetPacker(tkwin2))) {
+	    return TCL_OK;
+	}
 	if (packPtr->containerPtr == NULL) {
 	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		    "window \"%s\" isn't packed", argv2));
@@ -313,7 +319,9 @@ Tk_PackObjCmd(
 	    if (TkGetWindowFromObj(interp, tkwin, objv[i], &content) != TCL_OK) {
 		continue;
 	    }
-	    contentPtr = GetPacker(content);
+	    if (!(contentPtr = GetPacker(content))) {
+		continue;
+	    }
 	    if ((contentPtr != NULL) && (contentPtr->containerPtr != NULL)) {
 		Tk_ManageGeometry(content, NULL, NULL);
 		if (contentPtr->containerPtr->tkwin != Tk_Parent(contentPtr->tkwin)) {
@@ -338,7 +346,9 @@ Tk_PackObjCmd(
 	if (TkGetWindowFromObj(interp, tkwin, objv[2], &content) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	contentPtr = GetPacker(content);
+	if (!(contentPtr = GetPacker(content))) {
+	    return TCL_OK;
+	}
 	if (contentPtr->containerPtr == NULL) {
 	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		    "window \"%s\" isn't packed", argv2));
@@ -392,7 +402,9 @@ Tk_PackObjCmd(
 	if (TkGetWindowFromObj(interp, tkwin, objv[2], &container) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	containerPtr = GetPacker(container);
+	if (!(containerPtr = GetPacker(container))) {
+	    return TCL_OK;
+	}
 	if (objc == 3) {
 	    Tcl_SetObjResult(interp,
 		    Tcl_NewBooleanObj(!(containerPtr->flags & DONT_PROPAGATE)));
@@ -449,7 +461,9 @@ Tk_PackObjCmd(
 	    return TCL_ERROR;
 	}
 	resultObj = Tcl_NewObj();
-	containerPtr = GetPacker(container);
+	if (!(containerPtr = GetPacker(container))) {
+	    return TCL_OK;
+	}
 	for (contentPtr = containerPtr->contentPtr; contentPtr != NULL;
 		contentPtr = contentPtr->nextPtr) {
 	    Tcl_ListObjAppendElement(NULL, resultObj,
@@ -470,7 +484,7 @@ Tk_PackObjCmd(
 	    return TCL_ERROR;
 	}
 	packPtr = GetPacker(tkwin2);
-	if ((packPtr != NULL) && (packPtr->containerPtr != NULL)) {
+	if (packPtr && (packPtr->containerPtr != NULL)) {
 	    Tk_ManageGeometry(tkwin2, NULL, NULL);
 	    if (packPtr->containerPtr->tkwin != Tk_Parent(packPtr->tkwin)) {
 		Tk_UnmaintainGeometry(packPtr->tkwin,
@@ -1011,11 +1025,12 @@ YExpansion(
  * GetPacker --
  *
  *	This internal function is used to locate a Packer structure for a
- *	given window, creating one if one doesn't exist already.
+ *	window, creating one if one doesn't exist already, except if the window
+ *	is already dead.
  *
  * Results:
  *	The return value is a pointer to the Packer structure corresponding to
- *	tkwin.
+ *	tkwin, or NULL when tkwin is already dead.
  *
  * Side effects:
  *	A new packer structure may be created. If so, then a callback is set
@@ -1034,6 +1049,10 @@ GetPacker(
     int isNew;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
+    if (((TkWindow *) tkwin)->flags & TK_ALREADY_DEAD) {
+	return NULL;
+    }
+
     if (!dispPtr->packInit) {
 	dispPtr->packInit = 1;
 	Tcl_InitHashTable(&dispPtr->packerHashTable, TCL_ONE_WORD_KEYS);
@@ -1149,7 +1168,9 @@ PackAfter(
 	if (tkwin == containerPtr->tkwin) {
 	    goto badWindow;
 	}
-	packPtr = GetPacker(tkwin);
+	if (!(packPtr = GetPacker(tkwin))) {
+	    return TCL_OK;
+	}
 
 	/*
 	 * Process options for this window.
@@ -1570,7 +1591,10 @@ ConfigureContent(
 	    Tcl_SetErrorCode(interp, "TK", "GEOMETRY", "TOPLEVEL", NULL);
 	    return TCL_ERROR;
 	}
-	contentPtr = GetPacker(content);
+	if (!(contentPtr = GetPacker(content))) {
+	    continue;
+	}
+
 	contentPtr->flags &= ~OLD_STYLE;
 
 	/*
@@ -1608,7 +1632,9 @@ ConfigureContent(
 			    != TCL_OK) {
 			return TCL_ERROR;
 		    }
-		    prevPtr = GetPacker(other);
+		    if (!(prevPtr = GetPacker(other))) {
+			continue;
+		    }
 		    if (prevPtr->containerPtr == NULL) {
 		    notPacked:
 			Tcl_SetObjResult(interp, Tcl_ObjPrintf(
@@ -1634,7 +1660,9 @@ ConfigureContent(
 			    != TCL_OK) {
 			return TCL_ERROR;
 		    }
-		    otherPtr = GetPacker(other);
+		    if (!(otherPtr = GetPacker(other))) {
+			continue;
+		    }
 		    if (otherPtr->containerPtr == NULL) {
 			goto notPacked;
 		    }
@@ -1683,7 +1711,9 @@ ConfigureContent(
 			    != TCL_OK) {
 			return TCL_ERROR;
 		    }
-		    containerPtr = GetPacker(other);
+		    if (!(containerPtr = GetPacker(other))) {
+			continue;
+		    }
 		    prevPtr = containerPtr->contentPtr;
 		    if (prevPtr != NULL) {
 			while (prevPtr->nextPtr != NULL) {
@@ -1766,7 +1796,9 @@ ConfigureContent(
 	 */
 
 	if (!positionGiven) {
-	    containerPtr = GetPacker(Tk_Parent(content));
+	    if (!(containerPtr = GetPacker(Tk_Parent(content)))) {
+		continue;
+	    }
 	    prevPtr = containerPtr->contentPtr;
 	    if (prevPtr != NULL) {
 		while (prevPtr->nextPtr != NULL) {
