@@ -11,8 +11,9 @@
  */
 
 #include "tkWinInt.h"
+#include "tkBusy.h"
 
-typedef struct ThreadSpecificData {
+typedef struct {
     int initialized;		/* 0 means table below needs initializing. */
     Tcl_HashTable windowTable;  /* The windowTable maps from HWND to Tk_Window
 				 * handles. */
@@ -48,7 +49,7 @@ Tk_AttachHWND(
     Tk_Window tkwin,
     HWND hwnd)
 {
-    int new;
+    int isNew;
     Tcl_HashEntry *entryPtr;
     TkWinDrawable *twdPtr = (TkWinDrawable *) Tk_WindowId(tkwin);
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
@@ -65,12 +66,12 @@ Tk_AttachHWND(
      */
 
     if (twdPtr == NULL) {
-	twdPtr = (TkWinDrawable *) ckalloc(sizeof(TkWinDrawable));
+	twdPtr = (TkWinDrawable *)ckalloc(sizeof(TkWinDrawable));
 	twdPtr->type = TWD_WINDOW;
 	twdPtr->window.winPtr = (TkWindow *) tkwin;
     } else if (twdPtr->window.handle != NULL) {
 	entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable,
-		(char *)twdPtr->window.handle);
+		twdPtr->window.handle);
 	Tcl_DeleteHashEntry(entryPtr);
     }
 
@@ -79,8 +80,8 @@ Tk_AttachHWND(
      */
 
     twdPtr->window.handle = hwnd;
-    entryPtr = Tcl_CreateHashEntry(&tsdPtr->windowTable, (char *)hwnd, &new);
-    Tcl_SetHashValue(entryPtr, (ClientData)tkwin);
+    entryPtr = Tcl_CreateHashEntry(&tsdPtr->windowTable, (char *)hwnd, &isNew);
+    Tcl_SetHashValue(entryPtr, tkwin);
 
     return (Window)twdPtr;
 }
@@ -114,7 +115,7 @@ Tk_HWNDToWindow(
 	Tcl_InitHashTable(&tsdPtr->windowTable, TCL_ONE_WORD_KEYS);
 	tsdPtr->initialized = 1;
     }
-    entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable, (char *) hwnd);
+    entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable, hwnd);
     if (entryPtr != NULL) {
 	return (Tk_Window) Tcl_GetHashValue(entryPtr);
     }
@@ -169,15 +170,7 @@ TkpPrintWindowId(
 {
     HWND hwnd = (window) ? Tk_GetHWND(window) : 0;
 
-    /*
-     * Use pointer representation, because Win64 is P64 (*not* LP64). Windows
-     * doesn't print the 0x for %p, so we do it.
-     * bug #2026405: cygwin does output 0x for %p so test and recover.
-     */
-
-    sprintf(buf, "0x%p", hwnd);
-    if (buf[2] == '0' && buf[3] == 'x')
-	sprintf(buf, "%p", hwnd);
+    snprintf(buf, TCL_INTEGER_SPACE, "0x%" TCL_Z_MODIFIER "x", (size_t)hwnd);
 }
 
 /*
@@ -204,12 +197,15 @@ TkpPrintWindowId(
 int
 TkpScanWindowId(
     Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
-    CONST char *string,		/* String containing a (possibly signed)
+    const char *string,		/* String containing a (possibly signed)
 				 * integer in a form acceptable to strtol. */
     Window *idPtr)		/* Place to store converted result. */
 {
     Tk_Window tkwin;
-    Window number, *numberPtr = &number;
+    union {
+	HWND hwnd;
+	int number;
+    } win;
 
     /*
      * We want sscanf for the 64-bit check, but if that doesn't work, then
@@ -218,13 +214,13 @@ TkpScanWindowId(
 
     if (
 #ifdef _WIN64
-	    (sscanf(string, "0x%p", &number) != 1) &&
+	    (sscanf(string, "0x%p", &win.hwnd) != 1) &&
 #endif
-	    Tcl_GetInt(interp, string, (int *) numberPtr) != TCL_OK) {
+	    Tcl_GetInt(interp, string, &win.number) != TCL_OK) {
 	return TCL_ERROR;
     }
 
-    tkwin = Tk_HWNDToWindow((HWND) number);
+    tkwin = Tk_HWNDToWindow(win.hwnd);
     if (tkwin) {
 	*idPtr = Tk_WindowId(tkwin);
     } else {
@@ -272,7 +268,7 @@ TkpMakeWindow(
      * order.
      */
 
-    hwnd = CreateWindowEx(WS_EX_NOPARENTNOTIFY, TK_WIN_CHILD_CLASS_NAME, NULL,
+    hwnd = CreateWindowExW(WS_EX_NOPARENTNOTIFY, TK_WIN_CHILD_CLASS_NAME, NULL,
 	    (DWORD) style, Tk_X(winPtr), Tk_Y(winPtr), Tk_Width(winPtr),
 	    Tk_Height(winPtr), parentWin, NULL, Tk_GetHINSTANCE(), NULL);
     SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0,
@@ -297,7 +293,7 @@ TkpMakeWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDestroyWindow(
     Display *display,
     Window w)
@@ -309,7 +305,7 @@ XDestroyWindow(
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     /*
      * Remove references to the window in the pointer module then release the
@@ -318,12 +314,12 @@ XDestroyWindow(
 
     TkPointerDeadWindow(winPtr);
 
-    entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable, (char*)hwnd);
+    entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable, hwnd);
     if (entryPtr != NULL) {
 	Tcl_DeleteHashEntry(entryPtr);
     }
 
-    ckfree((char *)twdPtr);
+    ckfree(twdPtr);
 
     /*
      * Don't bother destroying the window if we are going to destroy the
@@ -333,6 +329,7 @@ XDestroyWindow(
     if (hwnd != NULL && !(winPtr->flags & TK_DONT_DESTROY_WINDOW)) {
 	DestroyWindow(hwnd);
     }
+    return Success;
 }
 
 /*
@@ -343,7 +340,7 @@ XDestroyWindow(
  *	Cause the given window to become visible.
  *
  * Results:
- *	None
+ *	Always returns Success or BadWindow.
  *
  * Side effects:
  *	Causes the window state to change, and generates a MapNotify event.
@@ -351,16 +348,20 @@ XDestroyWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XMapWindow(
     Display *display,
     Window w)
 {
     XEvent event;
     TkWindow *parentPtr;
-    TkWindow *winPtr = TkWinGetWinPtr(w);
+    TkWindow *winPtr;
 
-    display->request++;
+    if (!w) {
+	return BadWindow;
+    }
+    winPtr = TkWinGetWinPtr(w);
+    LastKnownRequestProcessed(display)++;
 
     ShowWindow(Tk_GetHWND(w), SW_SHOWNORMAL);
     winPtr->flags |= TK_MAPPED;
@@ -375,7 +376,7 @@ XMapWindow(
 	for (parentPtr = winPtr->parentPtr; ;
 		parentPtr = parentPtr->parentPtr) {
 	    if ((parentPtr == NULL) || !(parentPtr->flags & TK_MAPPED)) {
-		return;
+		return Success;
 	    }
 	    if (parentPtr->flags & TK_TOP_HIERARCHY) {
 		break;
@@ -383,7 +384,7 @@ XMapWindow(
 	}
     } else {
 	event.type = MapNotify;
-	event.xmap.serial = display->request;
+	event.xmap.serial = LastKnownRequestProcessed(display);
 	event.xmap.send_event = False;
 	event.xmap.display = display;
 	event.xmap.event = winPtr->window;
@@ -398,12 +399,13 @@ XMapWindow(
      */
 
     event.type = VisibilityNotify;
-    event.xvisibility.serial = display->request;
+    event.xvisibility.serial = LastKnownRequestProcessed(display);
     event.xvisibility.send_event = False;
     event.xvisibility.display = display;
     event.xvisibility.window = winPtr->window;
     event.xvisibility.state = VisibilityUnobscured;
     NotifyVisibility(&event, winPtr);
+    return Success;
 }
 
 /*
@@ -451,7 +453,7 @@ NotifyVisibility(
  *	Cause the given window to become invisible.
  *
  * Results:
- *	None
+ *	Always returns Success or BadWindow.
  *
  * Side effects:
  *	Causes the window state to change, and generates an UnmapNotify event.
@@ -459,15 +461,19 @@ NotifyVisibility(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XUnmapWindow(
     Display *display,
     Window w)
 {
     XEvent event;
-    TkWindow *winPtr = TkWinGetWinPtr(w);
+    TkWindow *winPtr;
 
-    display->request++;
+    if (!w) {
+	return BadWindow;
+    }
+    winPtr = TkWinGetWinPtr(w);
+    LastKnownRequestProcessed(display)++;
 
     /*
      * Bug fix: Don't short circuit this routine based on TK_MAPPED because it
@@ -479,7 +485,7 @@ XUnmapWindow(
 
     if (winPtr->flags & TK_WIN_MANAGED) {
 	event.type = UnmapNotify;
-	event.xunmap.serial = display->request;
+	event.xunmap.serial = LastKnownRequestProcessed(display);
 	event.xunmap.send_event = False;
 	event.xunmap.display = display;
 	event.xunmap.event = winPtr->window;
@@ -487,6 +493,7 @@ XUnmapWindow(
 	event.xunmap.from_configure = False;
 	Tk_HandleEvent(&event);
     }
+    return Success;
 }
 
 /*
@@ -505,15 +512,16 @@ XUnmapWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XMoveResizeWindow(
     Display *display,
     Window w,
     int x, int y,		/* Position relative to parent. */
     unsigned int width, unsigned int height)
 {
-    display->request++;
+    LastKnownRequestProcessed(display)++;
     MoveWindow(Tk_GetHWND(w), x, y, (int) width, (int) height, TRUE);
+    return Success;
 }
 
 /*
@@ -532,7 +540,7 @@ XMoveResizeWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XMoveWindow(
     Display *display,
     Window w,
@@ -540,10 +548,11 @@ XMoveWindow(
 {
     TkWindow *winPtr = TkWinGetWinPtr(w);
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     MoveWindow(Tk_GetHWND(w), x, y, winPtr->changes.width,
 	    winPtr->changes.height, TRUE);
+    return Success;
 }
 
 /*
@@ -562,7 +571,7 @@ XMoveWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XResizeWindow(
     Display *display,
     Window w,
@@ -570,16 +579,17 @@ XResizeWindow(
 {
     TkWindow *winPtr = TkWinGetWinPtr(w);
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     MoveWindow(Tk_GetHWND(w), winPtr->changes.x, winPtr->changes.y, (int)width,
 	    (int)height, TRUE);
+    return Success;
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * XRaiseWindow --
+ * XRaiseWindow, XLowerWindow --
  *
  *	Change the stacking order of a window.
  *
@@ -592,15 +602,28 @@ XResizeWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XRaiseWindow(
     Display *display,
     Window w)
 {
     HWND window = Tk_GetHWND(w);
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
     SetWindowPos(window, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    return Success;
+}
+
+int
+XLowerWindow(
+    Display *display,
+    Window w)
+{
+    HWND window = Tk_GetHWND(w);
+
+    LastKnownRequestProcessed(display)++;
+    SetWindowPos(window, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    return Success;
 }
 
 /*
@@ -622,7 +645,7 @@ XRaiseWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XConfigureWindow(
     Display *display,
     Window w,
@@ -632,7 +655,7 @@ XConfigureWindow(
     TkWindow *winPtr = TkWinGetWinPtr(w);
     HWND hwnd = Tk_GetHWND(w);
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     /*
      * Change the shape and/or position of the window.
@@ -657,6 +680,7 @@ XConfigureWindow(
 	}
 	TkWinSetWindowPos(hwnd, sibling, values->stack_mode);
     }
+    return Success;
 }
 
 /*
@@ -675,7 +699,7 @@ XConfigureWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XClearWindow(
     Display *display,
     Window w)
@@ -687,10 +711,10 @@ XClearWindow(
     HWND hwnd = Tk_GetHWND(w);
     HDC dc = GetDC(hwnd);
 
-    palette = TkWinGetPalette(display->screens[0].cmap);
+    palette = TkWinGetPalette(DefaultColormapOfScreen(ScreenOfDisplay(display, 0)));
     oldPalette = SelectPalette(dc, palette, FALSE);
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     winPtr = TkWinGetWinPtr(w);
     brush = CreateSolidBrush(winPtr->atts.background_pixel);
@@ -703,6 +727,7 @@ XClearWindow(
     DeleteObject(brush);
     SelectPalette(dc, oldPalette, TRUE);
     ReleaseDC(hwnd, dc);
+    return Success;
 }
 
 /*
@@ -723,7 +748,7 @@ XClearWindow(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XChangeWindowAttributes(
     Display *display,
     Window w,
@@ -733,6 +758,34 @@ XChangeWindowAttributes(
     if (valueMask & CWCursor) {
 	XDefineCursor(display, w, attributes->cursor);
     }
+    return Success;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * XReparentWindow --
+ *
+ *	TODO: currently placeholder to satisfy Xlib stubs.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	TODO.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+XReparentWindow(
+    Display *display,
+    Window w,
+    Window parent,
+    int x,
+    int y)
+{
+    return BadWindow;
 }
 
 /*
@@ -784,27 +837,174 @@ TkWinSetWindowPos(
 /*
  *----------------------------------------------------------------------
  *
- * TkpWindowWasRecentlyDeleted --
+ * TkpShowBusyWindow --
  *
- *	Determines whether we know if the window given as argument was
- *	recently deleted. Called by the generic code error handler to handle
- *	BadWindow events.
+ *	Makes a busy window "appear".
  *
  * Results:
- *	Always 0. We do not keep this information on Windows.
+ *	None.
  *
  * Side effects:
- *	None.
+ *	Arranges for the busy window to start intercepting events and the
+ *	cursor to change to the configured "hey, I'm busy!" setting.
  *
  *----------------------------------------------------------------------
  */
 
-int
-TkpWindowWasRecentlyDeleted(
-    Window win,
-    TkDisplay *dispPtr)
+void
+TkpShowBusyWindow(
+    TkBusy busy)
 {
-    return 0;
+    Busy *busyPtr = (Busy *) busy;
+    HWND hWnd;
+    POINT point;
+    Display *display;
+    Window window;
+
+    if (busyPtr->tkBusy != NULL) {
+	Tk_MapWindow(busyPtr->tkBusy);
+	window = Tk_WindowId(busyPtr->tkBusy);
+	display = Tk_Display(busyPtr->tkBusy);
+	hWnd = Tk_GetHWND(window);
+	LastKnownRequestProcessed(display)++;
+	SetWindowPos(hWnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    }
+
+    /*
+     * Under Win32, cursors aren't associated with windows. Tk fakes this by
+     * watching Motion events on its windows. So Tk will automatically change
+     * the cursor when the pointer enters the Busy window. But Windows does
+     * not immediately change the cursor; it waits for the cursor position to
+     * change or a system call. We need to change the cursor before the
+     * application starts processing, so set the cursor position redundantly
+     * back to the current position.
+     */
+
+    GetCursorPos(&point);
+    TkSetCursorPos(point.x, point.y);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpHideBusyWindow --
+ *
+ *	Makes a busy window "disappear".
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Arranges for the busy window to stop intercepting events, and the
+ *	cursor to change back to its normal setting.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpHideBusyWindow(
+    TkBusy busy)
+{
+    Busy *busyPtr = (Busy *) busy;
+    POINT point;
+
+    if (busyPtr->tkBusy != NULL) {
+	Tk_UnmapWindow(busyPtr->tkBusy);
+    }
+
+    /*
+     * Under Win32, cursors aren't associated with windows. Tk fakes this by
+     * watching Motion events on its windows. So Tk will automatically change
+     * the cursor when the pointer enters the Busy window. But Windows does
+     * not immediately change the cursor: it waits for the cursor position to
+     * change or a system call. We need to change the cursor before the
+     * application starts processing, so set the cursor position redundantly
+     * back to the current position.
+     */
+
+    GetCursorPos(&point);
+    TkSetCursorPos(point.x, point.y);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpMakeTransparentWindowExist --
+ *
+ *	Construct the platform-specific resources for a transparent window.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Moves the specified window in the stacking order.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpMakeTransparentWindowExist(
+    Tk_Window tkwin,		/* Token for window. */
+    Window parent)		/* Parent window. */
+{
+    TkWindow *winPtr = (TkWindow *) tkwin;
+    HWND hParent = (HWND) parent, hWnd;
+    int style = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
+    DWORD exStyle = WS_EX_TRANSPARENT | WS_EX_TOPMOST;
+
+    hWnd = CreateWindowExW(exStyle, TK_WIN_CHILD_CLASS_NAME, NULL, style,
+	    Tk_X(tkwin), Tk_Y(tkwin), Tk_Width(tkwin), Tk_Height(tkwin),
+	    hParent, NULL, Tk_GetHINSTANCE(), NULL);
+    winPtr->window = Tk_AttachHWND(tkwin, hWnd);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpCreateBusy --
+ *
+ *	Construct the platform-specific parts of a busy window. Note that this
+ *	postpones the actual creation of the window resource until later.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Sets up part of the busy window structure.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpCreateBusy(
+    Tk_FakeWin *winPtr,
+    Tk_Window tkRef,
+    Window *parentPtr,
+    Tk_Window tkParent,
+    TkBusy busy)
+{
+    Busy *busyPtr = (Busy *) busy;
+
+    if (winPtr->flags & TK_REPARENTED) {
+	/*
+	 * This works around a bug in the implementation of menubars for
+	 * non-Macintosh window systems (Win32 and X11). Tk doesn't reset the
+	 * pointers to the parent window when the menu is reparented
+	 * (winPtr->parentPtr points to the wrong window). We get around this
+	 * by determining the parent via the native API calls.
+	 */
+
+	HWND hWnd = GetParent(Tk_GetHWND(Tk_WindowId(tkRef)));
+	RECT rect;
+
+	if (GetWindowRect(hWnd, &rect)) {
+	    busyPtr->width = rect.right - rect.left;
+	    busyPtr->height = rect.bottom - rect.top;
+	}
+    } else {
+	*parentPtr = Tk_WindowId(tkParent);
+	*parentPtr = (Window) Tk_GetHWND(*parentPtr);
+    }
 }
 
 /*
